﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.FileProviders;
using System.Collections.Generic;
using System.Net;
using System.Net.Mime;
using System.Threading.Tasks;

using TestProject.Data;
using TestProject.Models;

namespace TestProject.Pages
{
    /// <summary>
    /// 인덱스 모델
    /// </summary>
    public class IndexModel : PageModel
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// DB 컨텍스트
        /// </summary>
        private readonly DatabaseContext dbContext;

        /// <summary>
        /// 파일 제공자
        /// </summary>
        private readonly IFileProvider fileProvider;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 파일 항목 리스트 - FileItemList

        /// <summary>
        /// 파일 항목 리스트
        /// </summary>
        public IList<FileItemModel> FileItemList { get; private set; }

        #endregion
        #region 디렉토리 컨텐츠 - DirectoryContents

        /// <summary>
        /// 디렉토리 컨텐츠
        /// </summary>
        public IDirectoryContents DirectoryContents { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - IndexModel(dbContext, fileProvider)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="dbContext">DB 컨텍스트</param>
        /// <param name="fileProvider">파일 공급자</param>
        public IndexModel(DatabaseContext dbContext, IFileProvider fileProvider)
        {
            this.dbContext    = dbContext;
            this.fileProvider = fileProvider;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region GET 요청시 처리하기 (비동기) - OnGetAsync()

        /// <summary>
        /// GET 처리하기 (비동기)
        /// </summary>
        /// <returns>태스크</returns>
        public async Task OnGetAsync()
        {
            FileItemList      = await this.dbContext.FileItemDBSet.AsNoTracking().ToListAsync();
            DirectoryContents = this.fileProvider.GetDirectoryContents(string.Empty);
        }

        #endregion
        #region 데이터베이스에서 다운로드 GET 요청시 처리하기 (비동기) - OnGetDownloadFromDatabaseAsync(id)

        /// <summary>
        /// 데이터베이스에서 다운로드 GET 요청시 처리하기 (비동기)
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>액션 결과 태스크</returns>
        public async Task<IActionResult> OnGetDownloadFromDatabaseAsync(int? id)
        {
            if(id == null)
            {
                return Page();
            }

            FileItemModel fileItem = await this.dbContext.FileItemDBSet.SingleOrDefaultAsync(m => m.ID == id);

            if(fileItem == null)
            {
                return Page();
            }

            // UI에 신뢰할 수 없는 파일 이름을 표시하지 마십시오.
            // 값을 HTML로 인코딩합니다.
            return File(fileItem.Content, MediaTypeNames.Application.Octet, WebUtility.HtmlEncode(fileItem.FileName));
        }

        #endregion
        #region 물리적 저장소에서 다운로드 GET 요청시 처리하기 - OnGetDownloadFromPhysicalStorage(fileName)

        /// <summary>
        /// 물리적 저장소에서 다운로드 GET 요청시 처리하기
        /// </summary>
        /// <param name="fileName">파일명</param>
        /// <returns>액션 결과</returns>
        public IActionResult OnGetDownloadFromPhysicalStorage(string fileName)
        {
            IFileInfo fileInfo = this.fileProvider.GetFileInfo(fileName);

            return PhysicalFile(fileInfo.PhysicalPath, MediaTypeNames.Application.Octet, fileName);
        }

        #endregion
    }
}